function [Q , c] = MVE(P, Fehler)

% Bemerkungen:
% Minimum Volume Covering Elliopsoid fr die Datenpunkte, die als Spaten 
% in der Matrix P gegeben sind. Dabei wird den Khachiyans Algorithmus
% verwendet. 
% Zugrundeliegende Problem ist:
%
% min (-ln det Q)
% subject to     (P_i - c)' * Q * (P_i - c) <= 1
%                
% Q ist die Strukturmatrix und c das Zentrum des gesuchten Ellipsoids. 
% P_i ist der i-te Datenpunkt als i-te Spalte von P.
% Dieser iterative Algorithmus brcksichtigt einen akzeptierten 'Fehler',
% mit dem das Ergebnis vom Optimum abweichen darf.
%
% Inputs:
% ===========================================================
% P : (N x T) dimnesionale Matrix mit T Datenpunkten von R^N.
% Fehler : akzeptierter Fehler.
%
% Outputs:
% ===========================================================
% Q : (N x N) Strukturmatrix des Ellipsoids 
% c : (N x 1) Vektor als Zentrum des Ellipsoids. 
% 
% Beispiel:
% ===========================================================
%      P = rand(5,100);
%      [Q, c] = MVE(P, .01)
%
%
% basiert auf der Arbeit von Nima Moshtagh (nima@seas.upenn.edu)
% University of Pennsylvania
% von December 2005 und UPDATE: Jan 2009
% http://www.mathworks.com/matlabcentral/fileexchange/9542


%% %%%%%%%%%%%%%%%%%%%% das duale Problem %%%%%%%%%%%%%%%%%%%%%%%%%%%
% ===========================================================
% Lifting der Datenpunkte
% ===========================================================
[N T] = size(P);
A = zeros(N+1,T);
A(1:N,:) = P(1:N,1:T);
A(N+1,:) = ones(1,T);

% Inititialisierung der Variablen
count = 1;
err = N+2;
u = (1/T) * ones(T,1);          % erste iteration

%% Khachiyans Algorithm
% -----------------------------------
Fehler=(N+1)*(1+Fehler);   % epsilon-Optimalitt nach Khachiyan

while err > Fehler,
    X = A * diag(u) * A';       % X = \sum_i ( u_i * a_i * a_i')  ist eine (N+1)x(N+1) Matrix
    M = diag(A' * (X\ A));     % M beinhaltet die aktuellen, quadratischen Q-Normen der T Datenpunkte
    [err j] = max(M);           % err ist die maximale Distanz der T Datenpunkte vom Zentrum
    schrittweite = (err - N -1)/((N+1)*(err-1));
    u_neu = (1 - schrittweite)*u ;
    u_neu(j) = u_neu(j) + schrittweite;
    count = count + 1;    
    u = u_neu;
end


%%%%%%%%%%%%%%%%%%% Umkehrung des Liftings %%%%%%%%%%%%%%%%%%%%%%
% Fr die Ausgabe der Strukturmatrix der ursprnglichen Datenpunkte:
% (x-c)' * Q * (x-c) = 1
% Nun wird Q als eine NxN Matrix und N dimensionale Zentrumsvektor c von
% dem epsilon-optimalen Ellipsoids berechnet.
U = diag(u);

% Strukturmatrix Q
Q = (1/N)*inv(P * U * P' - (P*u)*(P*u)' );

% Zentrum des Ellipsoids
c = P * u;

% Plotten der Distanzen
% plot(sort(diag((P-c*ones(1,T))' * Q * (P-c*ones(1,T))))) 